// Copyright 1998-2018 Epic Games, Inc. All Rights Reserved.

#include "IOSSmartlook.h"
#include "IOSSmartlookPrivatePCH.h"
#include "Modules/ModuleManager.h"

#include "Misc/Paths.h"
#include "Misc/ConfigCacheIni.h"

#import <SmartlookMetal/Smartlook.h>
#include "SmartlookMetal/SmartlookCBridge.h"

#include "Analytics.h"
#include "UObject/ObjectMacros.h"
#include "AnalyticsSettings.h"

IMPLEMENT_MODULE( FAnalyticsIOSSmartlook, IOSSmartlook )

TSharedPtr<IAnalyticsProvider> FAnalyticsProviderSmartlook::Provider;

void FAnalyticsIOSSmartlook::StartupModule()
{
	FString IniName = FString::Printf(TEXT("%sDefaultEngine.ini"), *FPaths::SourceConfigDir());
	FString AnalyticsSection = TEXT("Analytics");
	
	FString FSdkKey = FAnalytics::Get().GetConfigValueFromIni(IniName, AnalyticsSection, TEXT("SmartlookSdkKey"), true);
	NSString* SdkKey = [NSString stringWithFString:FSdkKey];
	
	FString FramerateString = FAnalytics::Get().GetConfigValueFromIni(IniName, AnalyticsSection, TEXT("SmartlookFramerate"), true);
	int32 Framerate = FCString::Atoi(*FramerateString);

	FString StartRecordingString = FAnalytics::Get().GetConfigValueFromIni(IniName, AnalyticsSection, TEXT("SmartlookStartRecording"), true);
	bool StartRecording = StartRecordingString.ToBool();

	[Smartlook setupWithKey:SdkKey options:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:Framerate] forKey:SLSetupOptionFramerateKey]];
	
	if (StartRecording) {
		[Smartlook startRecording];
	}
}

void FAnalyticsIOSSmartlook::ShutdownModule()
{
	FAnalyticsProviderSmartlook::Destroy();
}

static bool ConvertToBool(const FString& InString, bool bDefault)
{
	if (InString.Len() == 0)
	{
		return bDefault;
	}
	FString Result = InString.ToLower();
	return Result.Equals(TEXT("true")) || Result.Equals(TEXT("yes"));
}

TSharedPtr<IAnalyticsProvider> FAnalyticsIOSSmartlook::CreateAnalyticsProvider(const FAnalyticsProviderConfigurationDelegate& GetConfigValue) const
{
	if (GetConfigValue.IsBound())
	{
		const FString InAppToken = GetConfigValue.Execute(TEXT("SmartlookAppToken"), true);
		return FAnalyticsProviderSmartlook::Create(InAppToken);
	}
	else
	{
		UE_LOG(LogAnalytics, Warning, TEXT("Smartlook::CreateAnalyticsProvider called with an unbound delegate"));
	}
	return nullptr;
}

// Provider

FAnalyticsProviderSmartlook::FAnalyticsProviderSmartlook(const FString InAppToken) :
AppToken(InAppToken)
{
#if WITH_Smartlook
	FString IniName = FString::Printf(TEXT("%sDefaultEngine.ini"), *FPaths::SourceConfigDir());
	EventMap.Empty();
	TArray<FString> EventNames;
	TArray<FString> EventTokens;
	int NameCount = GConfig->GetArray(TEXT("SmartlookAnalyticsEventMapping"), TEXT("EventNames"), EventNames, IniName);
	int TokenCount = GConfig->GetArray(TEXT("SmartlookAnalyticsEventMapping"), TEXT("EventTokens"), EventTokens, IniName);
	int Count = NameCount <= TokenCount ? NameCount : TokenCount;
	for (int Index = 0; Index < Count; ++Index)
	{
		EventMap.Add(EventNames[Index], EventTokens[Index]);
	}
	
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

FAnalyticsProviderSmartlook::~FAnalyticsProviderSmartlook()
{
	if (bHasSessionStarted)
	{
		EndSession();
	}
}

bool FAnalyticsProviderSmartlook::StartSession(const TArray<FAnalyticsEventAttribute>& Attributes)
{
#if WITH_Smartlook
	const int32 AttrCount = Attributes.Num();
	
	bHasSessionStarted = true;
	
	[Smartlook startRecording];
	
	return bHasSessionStarted;
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
	return false;
#endif
}

void FAnalyticsProviderSmartlook::EndSession()
{
#if WITH_Smartlook
	bHasSessionStarted = false;
	
	[Smartlook stopRecording];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::FlushEvents()
{
#if WITH_Smartlook
	
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::SetUserID(const FString& InUserID)
{
#if WITH_Smartlook
	UserId = InUserID;
	
	NSString* userIDString = nil;
	
	if (InUserID.Len() > 0) {
		userIDString = [NSString stringWithFString:InUserID];
	}
	
	[Smartlook setUserIdentifier:userIDString];
	
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

FString FAnalyticsProviderSmartlook::GetUserID() const
{
#if WITH_Smartlook
	return UserId;
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
	return FString();
#endif
}

FString FAnalyticsProviderSmartlook::GetSessionID() const
{
#if WITH_Smartlook
	FString Id = TEXT("unavailable");
	
	return Id;
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
	return FString();
#endif
}

bool FAnalyticsProviderSmartlook::SetSessionID(const FString& InSessionID)
{
#if WITH_Smartlook
	// Ignored
	UE_LOG(LogAnalytics, Display, TEXT("IOSSmartlook::SetSessionID - ignoring call"));
	return false;
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
	return false;
#endif
}

void FAnalyticsProviderSmartlook::RecordEvent(const FString& EventName, const TArray<FAnalyticsEventAttribute>& Attributes)
{
#if WITH_Smartlook
	if (EventName.Len() > 0) {
		NSString* eventNameString = [NSString stringWithFString : EventName];
		
		NSDictionary* propDict = nil;
		
		const int32 attrCount = Attributes.Num();
		if (attrCount > 0) {
			NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:attrCount];
			
			for (auto Attr : Attributes) {
				NSString* attrKey = [NSString stringWithFString : Attr.AttrName];
				NSString* attrValue = [NSString stringWithFString : Attr.ToString()];
				
				if (attrKey != nil && attrValue != nil) {
					[attributesMutable setObject:attrValue forKey:attrKey];
				}
			}
			
			propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
		}
		
		[Smartlook trackCustomEventWithName:eventNameString props:propDict];
	}
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::RecordItemPurchase(const FString& ItemId, const FString& Currency, int PerItemCost, int ItemQuantity)
{
#if WITH_Smartlook
	NSString* eventNameString = @"ItemPurchase";

	NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:4];

	if (ItemId.Len() > 0) {
		NSString* itemIDString = [NSString stringWithFString : ItemId];
		[attributesMutable setObject:itemIDString forKey:@"ItemID"];
	}

	if (Currency.Len() > 0) {
		NSString* currencyString = [NSString stringWithFString : Currency];
		[attributesMutable setObject:currencyString forKey:@"Currency"];
	}
	
	NSString* perItemCostString = [NSString stringWithFormat:@"%ld", (long)PerItemCost];
	[attributesMutable setObject:perItemCostString forKey:@"PerItemCost"];
	
	NSString* itemQuantityString = [NSString stringWithFormat:@"%ld", (long)ItemQuantity];
	[attributesMutable setObject:itemQuantityString forKey:@"ItemQuantity"];
	
	NSDictionary* propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
	
	[Smartlook trackCustomEventWithName:eventNameString props:propDict];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::RecordCurrencyPurchase(const FString& GameCurrencyType, int GameCurrencyAmount, const FString& RealCurrencyType, float RealMoneyCost, const FString& PaymentProvider)
{
#if WITH_Smartlook
	NSString* eventNameString = @"CurrencyPurchase";
	
	NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:5];
	
	if (GameCurrencyType.Len() > 0) {
		NSString* currencyTypeString = [NSString stringWithFString : GameCurrencyType];
		[attributesMutable setObject:currencyTypeString forKey:@"GameCurrencyType"];
	}

	NSString* gameCurrencyAmountString = [NSString stringWithFormat:@"%ld", (long)GameCurrencyAmount];
	[attributesMutable setObject:gameCurrencyAmountString forKey:@"GameCurrencyAmount"];

	if (RealCurrencyType.Len() > 0) {
		NSString* realCurrencyTypeString = [NSString stringWithFString : RealCurrencyType];
		[attributesMutable setObject:realCurrencyTypeString forKey:@"RealCurrencyType"];
	}

	NSString* realMoneyCostString = [NSString stringWithFormat:@"%lf", (double)RealMoneyCost];
	[attributesMutable setObject:realMoneyCostString forKey:@"RealMoneyCost"];

	if (PaymentProvider.Len() > 0) {
		NSString* paymentProviderString = [NSString stringWithFString : PaymentProvider];
		[attributesMutable setObject:paymentProviderString forKey:@"PaymentProvider"];
	}

	
	NSDictionary* propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
	
	[Smartlook trackCustomEventWithName:eventNameString props:propDict];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::RecordCurrencyGiven(const FString& GameCurrencyType, int GameCurrencyAmount)
{
#if WITH_Smartlook
	NSString* eventNameString = @"CurrencyGiven";
	
	NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:2];
	
	if (GameCurrencyType.Len() > 0) {
		NSString* gameCurrencyTypeString = [NSString stringWithFString : GameCurrencyType];
		[attributesMutable setObject:gameCurrencyTypeString forKey:@"GameCurrencyType"];
	}

	NSString* gameCurrencyAmountString = [NSString stringWithFormat:@"%ld", (long)GameCurrencyAmount];
	[attributesMutable setObject:gameCurrencyAmountString forKey:@"GameCurrencyAmount"];

	
	NSDictionary* propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
	
	[Smartlook trackCustomEventWithName:eventNameString props:propDict];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::RecordError(const FString& Error, const TArray<FAnalyticsEventAttribute>& EventAttrs)
{
#if WITH_Smartlook
	NSString* eventNameString = @"Error";
	
	const int32 errorAttrCount = EventAttrs.Num();
	
	NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:(errorAttrCount + 1)];
	
	if (Error.Len() > 0) {
		NSString* errorString = [NSString stringWithFString : Error];
		[attributesMutable setObject:errorString forKey:@"Error"];
	}
	
	if (errorAttrCount > 0) {
		for (auto Attr : EventAttrs) {
			NSString* attrKey = [NSString stringWithFString : Attr.AttrName];
			NSString* attrValue = [NSString stringWithFString : Attr.ToString()];
			
			if (attrKey != nil && attrValue != nil) {
				[attributesMutable setObject:attrValue forKey:attrKey];
			}
		}
	}
	
	NSDictionary* propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
	
	[Smartlook trackCustomEventWithName:eventNameString props:propDict];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}

void FAnalyticsProviderSmartlook::RecordProgress(const FString& ProgressType, const FString& ProgressHierarchy, const TArray<FAnalyticsEventAttribute>& EventAttrs)
{
#if WITH_Smartlook
	NSString* eventNameString = @"Progress";
	
	const int32 progressAttrCount = EventAttrs.Num();
	
	NSMutableDictionary* attributesMutable = [NSMutableDictionary dictionaryWithCapacity:(progressAttrCount + 2)];
	
	if (ProgressType.Len() > 0) {
		NSString* progressTypeString = [NSString stringWithFString : ProgressType];
		[attributesMutable setObject:progressTypeString forKey:@"ProgressType"];
	}

	if (ProgressHierarchy.Len() > 0) {
		NSString* progressHierarchyString = [NSString stringWithFString : ProgressHierarchy];
		[attributesMutable setObject:progressHierarchyString forKey:@"ProgressName"];
	}

	if (progressAttrCount > 0) {
		for (auto Attr : EventAttrs) {
			NSString* attrKey = [NSString stringWithFString : Attr.AttrName];
			NSString* attrValue = [NSString stringWithFString : Attr.ToString()];
			
			if (attrKey != nil && attrValue != nil) {
				[attributesMutable setObject:attrValue forKey:attrKey];
			}
		}
	}
	
	NSDictionary* propDict = [NSDictionary dictionaryWithDictionary:attributesMutable];
	
	[Smartlook trackCustomEventWithName:eventNameString props:propDict];
#else
	UE_LOG(LogAnalytics, Warning, TEXT("WITH_Smartlook=0. Are you missing the SDK?"));
#endif
}
